package com.zipcodewilmington.generic.group;

import com.zipcodewilmington.generic.identifiables.IdentifiableInterface;
import com.zipcodewilmington.generic.identifiables.Person;

import java.io.Serializable;
import java.lang.reflect.Field;
import java.lang.reflect.ParameterizedType;
import java.util.ArrayList;
import java.util.List;
import java.util.function.Predicate;
import java.util.stream.Collectors;

/**
 * @author leon on 06/12/2018.
 */
public class Group<
        TypeOfId extends Serializable,
        TypeOfEntity extends IdentifiableInterface<TypeOfId>>
        extends AbstractGroup<TypeOfId, TypeOfEntity> {

    List<TypeOfEntity> group;

    public Group() {
        this.group = new ArrayList<>();
    }

    @Override
    public Integer count() {
        return this.group.size();
    }

    @Override
    public void insert(TypeOfEntity object) {
        this.group.add(object);
    }

    @Override
    public void delete(TypeOfEntity object) {
        this.group.remove(object);
    }

    @Override
    public void delete(Serializable serializable) {
        while(this.group.iterator().hasNext()) {
            if (this.group.iterator().next().getIdentity() == serializable) {
                this.group.remove(this.group.iterator().next());
            }
        }
    }

    @Override
    public Boolean has(IdentifiableInterface object) {
        return this.group.contains(object);
    }

    @Override
    public Boolean has(Serializable serializable) {
            for (IdentifiableInterface i : this.group) {
                if (serializable instanceof Number) {
                    if (i.getIdentity() == serializable) {
                        return true;
                    }
                }
                if (!(serializable instanceof Number)) {
                    if (i.getIdentity().equals(serializable)) {
                        return true;
                    }
                }
            }
        return false;
    }

    @Override
    public List filter(Predicate<TypeOfEntity> predicate) {
        return this.group.stream().filter(predicate).collect(Collectors.toList());
    }

    @Override
    public Class getIdentityType() {
        try {
            Field groupField = Group.class.getDeclaredField("group");
            ParameterizedType groupType = (ParameterizedType) groupField.getGenericType();
            Class<?> groupClass = (Class<?>) groupType.getActualTypeArguments()[0];
            return groupClass;
        } catch (NoSuchFieldException e) {
            e.printStackTrace();
        }
        return null;
    }

    @Override
    public Class getIdentifiableType() {
        try {
            Field groupField = Group.class.getDeclaredField("group");
            ParameterizedType groupType = (ParameterizedType) groupField.getGenericType();
            Class<?> groupClass = (Class<?>) groupType.getActualTypeArguments()[0];
            return groupClass;
        } catch (NoSuchFieldException e) {
            e.printStackTrace();
        }
        return null;    }

}
